//+------------------------------------------------------------------+
//|                                                   SignalSVR.mqh |
//|                   Copyright 2009-2017, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#include <Expert\ExpertSignal.mqh>
#include <My\Cmlp-.mqh>
//+------------------------------------------------------------------+
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=Signals based on SVR-Loss Tuned Network.                   |
//| Type=SignalAdvanced                                              |
//| Name=SVR                                                        |
//| ShortName=SVR                                                   |
//| Class=CSignalSVR                                                |
//| Page=signal_svr                                                 |
//| Parameter=TrainSet,int,65,Training Set Size                      |
//| Parameter=Epochs,int,5,Epochs                                    |
//| Parameter=LearningRate,double,0.0005,Learning Rate               |
//| Parameter=InitialWeight,double,0.005,Default Weight              |
//| Parameter=InitialBias,double,0.0,Default Bias                    |
//| Parameter=SVR_Epsilon,double,0.005,SVR Epsilon                   |
//| Parameter=SVR_C,double,1.0,SVR C                                 |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//| Class CSignalSVR.                                                |
//| Purpose: Class of SVR-Loss Tuned Network.                        |
//|            Derives from class CExpertSignal.                     |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
#define  __MLP_SIGN_INPUTS 4
#define  __MLP_SIGN_HIDDEN 4
#define  __MLP_SIGN_OUTPUTS 1
int      __MLP_SIGN_ARCH[3] // network arch

=
{  __MLP_SIGN_INPUTS, __MLP_SIGN_HIDDEN, __MLP_SIGN_OUTPUTS
};
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CSignalSVR   : public CExpertSignal
{
protected:
   int                           m_train_set;                       //
   int                           m_epochs;                              // Epochs
   double                        m_learning_rate;                       // Epsilon

   double                        m_initial_weight;
   double                        m_initial_bias;

   Slearning                     m_learning;
   Smlp                          m_mlp;

   double                        m_train_deposit;
   double                        m_train_criteria;

   Eregularize                   m_regularization;
   ENUM_MATRIX_NORM              m_norm;
   double                        m_lambda;

   double                        m_svr_epsilon;
   double                        m_svr_c;


public:
                     CSignalSVR(void);
                    ~CSignalSVR(void);

   //--- methods of setting adjustable parameters
   void                          TrainSet(int value)
   {  m_train_set = value;
   }
   void                          Epochs(int value)
   {  m_epochs = value;
   }
   void                          LearningRate(double value)
   {  m_learning_rate = value;
   }
   
   void                          InitialWeight(double value)
   {  m_initial_weight = value;
   }
   void                          InitialBias(double value)
   {  m_initial_bias = value;
   }

   void                          SVR_Epsilon(double value)
   {  m_svr_epsilon = value;
   }
   void                          SVR_C(double value)
   {  m_svr_c = value;
   }

   //--- method of verification of arch
   virtual bool      ValidationSettings(void);
   //--- method of creating the indicator and timeseries
   virtual bool      InitIndicators(CIndicators *indicators);
   //--- methods of checking if the market models are formed
   virtual int       LongCondition(void);
   virtual int       ShortCondition(void);

protected:
   void                          GetOutput(vector &Output);
   void                          SetOutput();

   Cmlp                          *MLP;
};
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
void CSignalSVR::CSignalSVR(void) :    m_train_set(65),
   m_epochs(5),
   m_learning_rate(0.0005),
   m_initial_weight(0.005),
   m_initial_bias(0.0),
   m_svr_epsilon(0.005),
   m_svr_c(1.0)

{
//--- initialization of protected data
   m_used_series = USE_SERIES_OPEN + USE_SERIES_HIGH + USE_SERIES_LOW + USE_SERIES_CLOSE + USE_SERIES_SPREAD + USE_SERIES_TIME;
   ArrayResize(m_mlp.arch, ArraySize(__MLP_SIGN_ARCH));
   for(int i = 0; i < ArraySize(__MLP_SIGN_ARCH); i++)
   {  m_mlp.arch[i] = __MLP_SIGN_ARCH[i];
   }
   m_mlp.initial_weight = m_initial_weight;
   m_mlp.initial_bias = m_initial_bias;
   m_mlp.activation = AF_SOFTSIGN;
   m_mlp.loss = LOSS_HUBER;
   m_mlp.svr_loss = true;
   m_mlp.svr_epsilon = m_svr_epsilon;
   m_mlp.svr_c = m_svr_c;
//
   MLP = new Cmlp(m_mlp);
//
   m_learning.epochs = m_epochs;
   m_learning.initial_rate = m_learning_rate;
   m_learning.rate = m_learning_rate;
   m_learning.type = LEARNING_FIXED;
   m_train_deposit = AccountInfoDouble(ACCOUNT_BALANCE);
   m_train_criteria = 0.0;
}
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
void CSignalSVR::~CSignalSVR(void)
{  delete MLP;
}
//+------------------------------------------------------------------+
//| Validation arch protected data.                              |
//+------------------------------------------------------------------+
bool CSignalSVR::ValidationSettings(void)
{  if(!CExpertSignal::ValidationSettings())
      return(false);
//--- initial data checks
   if(m_epochs < 1)
   {  printf(__FUNCSIG__ + " epochs size should at least be 1! ");
      return(false);
   }
   if(m_learning_rate < 0.0)
   {  printf(__FUNCSIG__ + " learning rate should be greater than 0.0! ");
      return(false);
   }
   if(!MLP.validated)
   {  printf(__FUNCSIG__ + " invalid network arch. ");
      printf(__FUNCSIG__ +
             " invalid network arch! Settings size is: %i, Max layer size is: %i, Min layer size is: %i, and activation is %s ",
             ArraySize(m_mlp.arch), m_mlp.arch[ArrayMaximum(m_mlp.arch)], m_mlp.arch[ArrayMinimum(m_mlp.arch)], EnumToString(m_mlp.activation)
            );
      return(false);
   }
//read best weights
//--- ok
   return(true);
}
//+------------------------------------------------------------------+
//| Create indicators.                                               |
//+------------------------------------------------------------------+
bool CSignalSVR::InitIndicators(CIndicators *indicators)
{
//--- check pointer
   if(indicators == NULL)
      return(false);
//--- initialization of indicators and timeseries of additional filters
   if(!CExpertSignal::InitIndicators(indicators))
      return(false);
//--- ok
   return(true);
}
//+------------------------------------------------------------------+
//| "Voting" that price will grow.                                   |
//+------------------------------------------------------------------+
int CSignalSVR::LongCondition(void)
{  int result = 0;
   vector _out;
   GetOutput(_out);
   m_close.Refresh(-1);
   if(_out[0] > 0.0)
   {  result = int(round(100.0 * (fabs(_out[0]) / (fabs(_out[0]) + fabs(m_close.GetData(StartIndex()) - m_close.GetData(StartIndex() + 1))))));
   }
//printf(__FUNCSIG__ + " output is: %.5f,  and result is: %i", _out[0],  result);return(0);
   return(result);
}
//+------------------------------------------------------------------+
//| "Voting" that price will fall.                                   |
//+------------------------------------------------------------------+
int CSignalSVR::ShortCondition(void)
{  int result = 0;
   vector _out;
   GetOutput(_out);
   m_close.Refresh(-1);
   if(_out[0] < 0.0)
   {  result = int(round(100.0 * (fabs(_out[0]) / (fabs(_out[0]) + fabs(m_close.GetData(StartIndex()) - m_close.GetData(StartIndex() + 1))))));
   }
//printf(__FUNCSIG__ + " output is: %.5f,  and result is: %i", _out[0],  result);return(0);
   return(result);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CSignalSVR::GetOutput(vector &Output)
{  m_learning.rate = m_learning_rate;
   for(int i = m_epochs; i >= 1; i--)
   {  MLP.LearningType(m_learning, i);
      for(int ii = m_train_set; ii >= 0; ii--)
      {  vector _in, _in_new, _in_old;
         if
         (
            _in_new.Init(__MLP_SIGN_INPUTS) &&
            _in_new.CopyRates(m_symbol.Name(), m_period, 8, ii + __MLP_SIGN_OUTPUTS, __MLP_SIGN_INPUTS) &&
            _in_new.Size() == __MLP_SIGN_INPUTS &&
            _in_old.Init(__MLP_SIGN_INPUTS) &&
            _in_old.CopyRates(m_symbol.Name(), m_period, 8, ii + __MLP_SIGN_OUTPUTS + __MLP_SIGN_OUTPUTS, __MLP_SIGN_INPUTS) &&
            _in_old.Size() == __MLP_SIGN_INPUTS
         )
         {  _in = _in_new - _in_old;
            MLP.Set(_in);
            MLP.Forward();
            if(ii > 0)
            {  vector _target, _target_new, _target_old;
               if
               (
                  _target_new.Init(__MLP_SIGN_OUTPUTS) &&
                  _target_new.CopyRates(m_symbol.Name(), m_period, 8, ii, __MLP_SIGN_OUTPUTS) &&
                  _target_new.Size() == __MLP_SIGN_OUTPUTS &&
                  _target_old.Init(__MLP_SIGN_OUTPUTS) &&
                  _target_old.CopyRates(m_symbol.Name(), m_period, 8, ii + __MLP_SIGN_OUTPUTS, __MLP_SIGN_OUTPUTS) &&
                  _target_old.Size() == __MLP_SIGN_OUTPUTS
               )
               {  _target = _target_new - _target_old;
                  MLP.Get(_target);
                  MLP.Backward(m_learning, i);
               }
            }
            Output = MLP.output;
         }
      }
   }
}
//+------------------------------------------------------------------+
